// main.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "main.h"
#include "group.h"
#include "utils.h"
#include "bangs.h"
#include "DropTarget.h"
#include <iostream>

using namespace std;

extern "C" {
    __declspec( dllexport ) int initModuleEx(HWND hwndParent, HINSTANCE hDllInstance, LPCSTR szPath);
    __declspec( dllexport ) void quitModule(HINSTANCE hDllInstance);
}

#define CONFIG_TOKENS_NUM	2

const char g_rcsRevision[]	= "1.1.3";
const char g_szAppName[]	= "Clickonic";
const char g_szMsgHandler[]	= "LSClickonicManager";
const char g_szAuthor[]		= "Seg@, Andymon, Alurcard2";
const char g_szWClass[]		= "LSClickonicBGManager";

HINSTANCE g_hInstance;

int g_lsMessages[] = {LM_GETREVID, LM_REFRESH, 0};
HWND g_hwndMessageHandler;
void CreateMessageHandler(HINSTANCE hInst);
LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

// we are adding data to the beginning of the g_pGroups list
// since it is easier to implement
PGROUPLISTITEM g_pGroups = NULL;

///////////////////////////////////////////////////////////////////////////////
//
// DLL entry point
//
BOOL APIENTRY DllMain( HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved) {
	if (ul_reason_for_call == DLL_PROCESS_ATTACH)
		DisableThreadLibraryCalls((HINSTANCE)hModule);
	return TRUE;
}


///////////////////////////////////////////////////////////////////////////////
//
// Module entry point
//
int initModuleEx(HWND hwndParent, HINSTANCE hDllInstance, LPCSTR szPath) {

	CreateMessageHandler(hDllInstance);

	WNDCLASSEX wc;
	ZeroMemory(&wc, sizeof(WNDCLASSEX));
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.lpfnWndProc = Group_BGProc;
	wc.hInstance = hDllInstance;
	wc.lpszClassName = g_szWClass;
	wc.hIconSm = 0;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.style = CS_DBLCLKS;
	RegisterClassEx(&wc);

	g_hInstance = hDllInstance;

	OleInitialize(NULL);
	CoInitialize(NULL); // initialize COM

	InitCommonControls();
	CDropTarget::RegisterFormats();

	FILE *f;
	if (!(f = LCOpen(NULL))) {
		utils::ErrorBox("Error: Cannot open step.rc for reading");
		return 0;
	}

	char szLine[MAX_LINE_LENGTH], token[CONFIG_TOKENS_NUM][MAX_LINE_LENGTH],
		szExtra[MAX_LINE_LENGTH], szConfigLine[MAX_RCCOMMAND];
	char* tokens[CONFIG_TOKENS_NUM];

	int i;

	for (i = 0; i < CONFIG_TOKENS_NUM; i++)
		tokens[i] = token[i];

	StringCchPrintf(szConfigLine, MAX_RCCOMMAND, "*%s", g_szAppName);
	while(LCReadNextConfig(f, szConfigLine, szLine, MAX_LINE_LENGTH)) {
		LCTokenize(szLine, tokens, CONFIG_TOKENS_NUM, szExtra);
		if (token[1][0] == 0)
			utils::ErrorBoxPrint("Invalid config: *%s string without group name found", g_szAppName);
		else {
			CGroup *pGroup = new CGroup(token[1]);
			if (!pGroup->m_bInitialized)
				delete pGroup;
			else {
				PGROUPLISTITEM pItem = new GroupListItem;
				pItem->pGroup = pGroup;
				pItem->pNext = g_pGroups;
				g_pGroups = pItem;
			}
		}
	}
	LCClose(f);
	RegisterBangs();

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// Module destructor
//
void quitModule(HINSTANCE hDllInstance) {
	UnregisterBangs();

	while (g_pGroups) {
		PGROUPLISTITEM pFirst = g_pGroups;
		g_pGroups = g_pGroups->pNext;
		delete pFirst->pGroup;
		delete pFirst;
	}

	if (g_hwndMessageHandler) {
		SendMessage(GetLitestepWnd(),LM_UNREGISTERMESSAGE,(WPARAM)g_hwndMessageHandler, (LPARAM)g_lsMessages);
		DestroyWindow(g_hwndMessageHandler);
	}
	UnregisterClass(g_szMsgHandler, hDllInstance);
	UnregisterClass(g_szWClass, hDllInstance);

	CoUninitialize(); // shut down COM
	OleUninitialize();
}

///////////////////////////////////////////////////////////////////
//
// Find the icon group with the specified name
//
CGroup* GetGroupByName(const char *szName) {
	char szBuf[MAX_LINE_LENGTH];
	StringCchCopy(szBuf, MAX_LINE_LENGTH, szName);
	PathUnquoteSpaces(szBuf);
	PGROUPLISTITEM pEnumerator = g_pGroups;
	while (pEnumerator) {
		if (_stricmp(pEnumerator->pGroup->m_szName, szBuf) == 0)
			return pEnumerator->pGroup;
		pEnumerator = pEnumerator->pNext;
	}
	return NULL;
}


////////////////////////////////////////////////////////////////////////////
//
// Create message handler window
//

void CreateMessageHandler(HINSTANCE hInst) {
	WNDCLASSEX wc;
	ZeroMemory(&wc, sizeof(WNDCLASSEX));
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.lpfnWndProc = MessageHandlerProc;
	wc.hInstance = hInst;
	wc.lpszClassName = g_szMsgHandler;
	wc.hIconSm = 0;

	if (!RegisterClassEx(&wc)) {
		utils::ErrorBox("Error: Cannot register handler window class...");
		return;
	}

	g_hwndMessageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		g_szMsgHandler,
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInst,
		0);

	if (!g_hwndMessageHandler) {
		utils::ErrorBox("Error: Cannot create main window...");
		return;
	}	
	SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM)g_hwndMessageHandler, (LPARAM) g_lsMessages);
}


////////////////////////////////////////////////////////////////////////////
//
// Message handler process the following messages
//	LM_GETREVID				-	Revision ID
//
LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch(uMsg) {
		case LM_REFRESH: {
			// We should handle this.
			/*PGROUPLISTITEM pFirst = g_pGroups;
			while (pFirst) {
				pFirst->pGroup->ReadSettings(true);
				pFirst = pFirst->pNext;
			}*/
			return 0;
		}
		case LM_GETREVID: {
			size_t uLength;
			StringCchPrintf((char*)lParam, 64, "%s: %s", g_szAppName, g_rcsRevision);
			
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;

			lParam = NULL;
			return 0;
		}
		/* // This is a rather irrelevant function which is far more likely to crash litestep than do anything usefull
		case WM_SETTINGCHANGE: {
			if (wParam == 0 && !_stricmp("Windows", (char*)lParam)) {
				PGROUPLISTITEM pFirst = g_pGroups;
				while (pFirst) {
					pFirst->pGroup->HandleSettingChange();
					pFirst = pFirst->pNext;
				}
			}
			break;
		}*/
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

LRESULT WINAPI Group_BGProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
	CGroup* group = (CGroup *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
	if (!group)
		return DefWindowProc(hWnd, msg, wParam, lParam);
	switch (msg) {
		case WM_ERASEBKGND: {
			PaintDesktop((HDC)wParam);
			return 1;
		}
		case WM_WINDOWPOSCHANGING: {
			// Trap windowposchanging messages
			WINDOWPOS *c = (WINDOWPOS*)lParam;
			c->hwnd = hWnd;
			c->hwndInsertAfter = HWND_BOTTOM;
			c->x = group->m_nX;
			c->y = group->m_nY;
			c->cx = group->m_nWidth;
			c->cy = group->m_nHeight;
			c->flags = c->flags & ~SWP_NOZORDER;
			c->flags |= SWP_NOACTIVATE|SWP_NOSENDCHANGING;
			return 0;
		}
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

LRESULT WINAPI Group_Proc(HWND hListView, UINT msg, WPARAM wParam, LPARAM lParam) {
	CGroup* group = (CGroup *)GetWindowLongPtr(hListView, GWLP_USERDATA);
	if (!group)
		return DefWindowProc(hListView, msg, wParam, lParam);

	switch(msg) {
		case WM_NOTIFY: {
			switch (((LPNMHDR)lParam)->code) {
				case LVN_INSERTITEM: {
					NMLISTVIEW *nmListView = (LPNMLISTVIEW)lParam;
					if (group->HandleInsItem( nmListView->iItem ) && group->m_bInitialized && !group->m_bInFolderChange) {
						char szEvar[MAX_RCCOMMAND], szNum[10];
						sprintf_s(szEvar, MAX_RCCOMMAND, "%s%s", group->m_szName, "ItemCount");
						_itoa_s(ListView_GetItemCount(group->m_hwndListView), szNum, 10, 10);
						LSSetVariable(szEvar, szNum);
						group->HandleEvent(EVENT_ONINSERTITEM);
					}
					return 0;
				} // LVN_INSERTITEM
				case LVN_DELETEITEM: {
					NMLISTVIEW *nmListView = (LPNMLISTVIEW)lParam;
					char szFileName[MAX_PATH_LENGTH];

					group->GetNameFromId(nmListView->iItem, szFileName);

					if (!group->IconShouldBeHidden( szFileName ) && group->m_bInitialized && !group->m_bInFolderChange)
						group->HandleEvent(EVENT_ONREMOVEITEM);

					char szEvar[MAX_RCCOMMAND], szNum[10];	
					sprintf_s(szEvar, MAX_RCCOMMAND, "%s%s", group->m_szName, "ItemCount");
					_itoa_s(ListView_GetItemCount(group->m_hwndListView)-1, szNum, 10, 10);
					LSSetVariable(szEvar, szNum);

					break;
				} // LVN_DELETEITEM
			} // ((LPNMHDR)lParam)->code
			break;
		} // WM_NOTIFY
		case WM_DRAWCLIPBOARD: {
			group->ClearCutMark();
			SendMessage(group->m_hwndNextClipViewer, WM_DRAWCLIPBOARD, 0, 0);
			return 0;
		} // WM_DRAWCLIPBOARD:
		case WM_CHANGECBCHAIN: {
			if ((HWND)wParam == group->m_hwndNextClipViewer)
				group->m_hwndNextClipViewer = (HWND)lParam;
			else if (group->m_hwndNextClipViewer != NULL)
				SendMessage(group->m_hwndNextClipViewer, WM_CHANGECBCHAIN, wParam, lParam);
			return 0;
		} // WM_CHANGECBCHAIN
	} // msg
	return CallWindowProc(group->wpOrigGroup_Proc, hListView, msg, wParam, lParam);
}

LRESULT WINAPI ListView_Proc(HWND hListView, UINT msg, WPARAM wParam, LPARAM lParam) {
	CGroup* group = (CGroup *)GetWindowLongPtr(hListView, GWLP_USERDATA);
	if (!group)
		return DefWindowProc(hListView, msg, wParam, lParam);

	switch(msg) {
		case LVM_SETICONSPACING: {
			// Prevent explorers "Align to Grid" setting from messing up the spacing.
			return CallWindowProc(group->wpOrigListView_Proc, hListView, msg, wParam, (LPARAM)MAKELONG(group->m_nSpacingX, group->m_nSpacingY));
		}
		case LVM_SETEXTENDEDLISTVIEWSTYLE: {
			// Take some time to fix group variables, just in case this is called by an external app.
			/*DWORD dwExMask = (DWORD)wParam, dwExStyle = (DWORD)lParam;
			if (dwExMask == 0)
				dwExMask = dwExStyle;

			DWORD dwCurrentStyle = ListView_GetExtendedListViewStyle(group->m_hwndListView);
			
			// wtf, dwExMask is 0 even when it disables snap to grid...
			if ((dwExMask & LVS_EX_SNAPTOGRID) == LVS_EX_SNAPTOGRID) {
				if ((dwExStyle & LVS_EX_SNAPTOGRID) == LVS_EX_SNAPTOGRID)
					group->m_bSnapToGrid = true;
				else
					group->m_bSnapToGrid = false;
			}*/

			break;
		}
		case WM_KEYDOWN: {
			if (group->m_bInFolderChange)
				return 0;
			switch (wParam) {
				case VK_DELETE: {
					group->HandleEvent(EVENT_ONDELDOWN);
					return 0;
				}
				case VK_F1: {
					group->HandleEvent(EVENT_ONF1DOWN);
					return 0;
				}
				case VK_F2: {
					group->HandleEvent(EVENT_ONF2DOWN);
					return 0;
				}
				case VK_F3: {
					group->HandleEvent(EVENT_ONF3DOWN);
					return 0;
				}
				case VK_F4: {
					group->HandleEvent(EVENT_ONF4DOWN);
					return 0;
				}
				case VK_F5: {
					group->HandleEvent(EVENT_ONF5DOWN);
					return 0;
				}
				case VK_F6: {
					group->HandleEvent(EVENT_ONF6DOWN);
					return 0;
				}
				case VK_F7: {
					group->HandleEvent(EVENT_ONF7DOWN);
					return 0;
				}
				case VK_F8: {
					group->HandleEvent(EVENT_ONF8DOWN);
					return 0;
				}
				case VK_F9: {
					group->HandleEvent(EVENT_ONF9DOWN);
					return 0;
				}
				case VK_F10: {
					group->HandleEvent(EVENT_ONF10DOWN);
					return 0;
				}
				case VK_F11: {
					group->HandleEvent(EVENT_ONF11DOWN);
					return 0;
				}
				case VK_F12: {
					group->HandleEvent(EVENT_ONF12DOWN);
					return 0;
				}
				case VK_RETURN: {
					group->HandleEvent(EVENT_ONENTERDOWN);
					return 0;
				}
				case 0x41: { // A
					if (GetKeyState( VK_CONTROL ) < 0)
						group->HandleEvent(EVENT_ONCTRLA);
					break;
				}
				case 0x43: { // C
					if (GetKeyState( VK_CONTROL ) < 0)
						group->HandleEvent(EVENT_ONCTRLC);
					break;
				}
				case 0x58: { // X
					if (GetKeyState( VK_CONTROL ) < 0)
						group->HandleEvent(EVENT_ONCTRLX);
					break;
				}
				case 0x56: { // V
					if (GetKeyState( VK_CONTROL ) < 0)
						group->HandleEvent(EVENT_ONCTRLV);
					break;
				}
				case 0x5A: { // Z
					if (GetKeyState( VK_CONTROL ) < 0)
						group->HandleEvent(EVENT_ONCTRLZ);
					break;
				}
				case VK_LEFT:{
					group->HandleEvent(EVENT_ONLEFTKEYDOWN);
					return 0;
				}
				case VK_RIGHT: {
					group->HandleEvent(EVENT_ONRIGHTKEYDOWN);
					return 0;
				}
				case VK_UP: {
					group->HandleEvent(EVENT_ONUPKEYDOWN);
					return 0;
				}
				case VK_DOWN: {
					group->HandleEvent(EVENT_ONDOWNKEYDOWN);
					return 0;
				}
			} // wParam
			break;
		} // WM_KEYDOWN
		case WM_KEYUP: {
			switch (wParam) {
				case VK_RETURN: {
					group->HandleEvent(EVENT_ONENTERUP);
					return 0;
				}
				case VK_DELETE: {
					group->HandleEvent(EVENT_ONDELUP);
					return 0;
				}
				case VK_F1: {
					group->HandleEvent(EVENT_ONF1UP);
					return 0;
				}
				case VK_F2: {
					group->HandleEvent(EVENT_ONF2UP);
					return 0;
				}
				case VK_F3: {
					group->HandleEvent(EVENT_ONF3UP);
					return 0;
				}
				case VK_F4: {
					group->HandleEvent(EVENT_ONF4UP);
					return 0;
				}
				case VK_F5: {
					group->HandleEvent(EVENT_ONF5UP);
					return 0;
				}
				case VK_F6: {
					group->HandleEvent(EVENT_ONF6UP);
					return 0;
				}
				case VK_F7: {
					group->HandleEvent(EVENT_ONF7UP);
					return 0;
				}
				case VK_F8: {
					group->HandleEvent(EVENT_ONF8UP);
					return 0;
				}
				case VK_F9: {
					group->HandleEvent(EVENT_ONF9UP);
					return 0;
				}
				case VK_F10: {
					group->HandleEvent(EVENT_ONF10UP);
					return 0;
				}
				case VK_F11: {
					group->HandleEvent(EVENT_ONF11UP);
					return 0;
				}
				case VK_F12: {
					group->HandleEvent(EVENT_ONF12UP);
					return 0;
				}
				case VK_LEFT: {
					group->HandleEvent(EVENT_ONLEFTKEYUP);
					return 0;
				}
				case VK_RIGHT: {
					group->HandleEvent(EVENT_ONRIGHTKEYUP);
					return 0;
				}
				case VK_UP: {
					group->HandleEvent(EVENT_ONUPKEYUP);
					return 0;
				}
				case VK_DOWN: {
					group->HandleEvent(EVENT_ONDOWNKEYUP);
					return 0;
				}
			} // wParam
			break;
		} // WM_KEYUP
		case WM_MOUSEWHEEL: {
			if (GET_WHEEL_DELTA_WPARAM(wParam) > 0)
				group->HandleMouseEvent(EVENT_ONWHEELUP, msg, wParam, lParam);
			else
				group->HandleMouseEvent(EVENT_ONWHEELDOWN, msg, wParam, lParam);
			return 0;
		}
		case WM_MOUSEHWHEEL: {
			if (GET_WHEEL_DELTA_WPARAM(wParam) > 0)
				group->HandleMouseEvent(EVENT_ONWHEELRIGHT, msg, wParam, lParam);
			else
				group->HandleMouseEvent(EVENT_ONWHEELLEFT, msg, wParam, lParam);
			return 0;
		}
		case WM_LBUTTONDBLCLK: {
			if (group->m_bInlineBrowsing) {
				LVHITTESTINFO lvhi;
				ZeroMemory(&lvhi, sizeof(LVHITTESTINFO));
				lvhi.flags = LVHT_ONITEM;
				lvhi.pt.x = GET_X_LPARAM(lParam);
				lvhi.pt.y = GET_Y_LPARAM(lParam);
				int nFileid = ListView_HitTest(group->m_hwndListView, &lvhi);
				if ( nFileid != -1 ) {
					char szFileName[MAX_PATH_LENGTH];
					group->GetNameFromId(nFileid, szFileName);
					/*if (_stricmp(szFileName, "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}") == 0) {
						group->ChangeDir(".mycomputer");
						return 0;
					}*/
					if (utils::Is_Directory(szFileName)) {
						if (!group->m_bNoVirtualSwitch && (_stricmp(szFileName, group->m_szDesktopPath) == 0))
							group->ChangeDir(".desktop");
						else
							group->ChangeDir(szFileName);
						return 0;
					} // utils::Is_Directory(szFileName)
				} // nFileid != -1
			} // group->m_bInlineBrowsing
			group->HandleMouseEvent(EVENT_ONLEFTDBLCLICK, msg, wParam, lParam);
			return 0;
		} // WM_LBUTTONDBLCLK
		case WM_LBUTTONDOWN: {
			group->HandleMouseEvent(EVENT_ONLEFTCLICKDOWN, msg, wParam, lParam);
			return 0;
		}
		case WM_LBUTTONUP: {
			group->HandleMouseEvent(EVENT_ONLEFTCLICKUP, msg, wParam, lParam);
			return 0;
		}
		case WM_MBUTTONDBLCLK: {
			group->HandleMouseEvent(EVENT_ONMIDDLEDBLCLICK, msg, wParam, lParam);
			return 0;
		}
		case WM_MBUTTONDOWN: {
			group->HandleMouseEvent(EVENT_ONMIDDLECLICKDOWN, msg, wParam, lParam);
			return 0;
		}
		case WM_MBUTTONUP: {
			group->HandleMouseEvent(EVENT_ONMIDDLECLICKUP, msg, wParam, lParam);
			return 0;
		}
		case WM_XBUTTONDBLCLK: {
			if (HIWORD(wParam) == XBUTTON1)
				group->HandleMouseEvent(EVENT_ONX1DBLCLICK, msg, wParam, lParam);
			else if (HIWORD(wParam) == XBUTTON2) // There could technically be more xbuttons
				group->HandleMouseEvent(EVENT_ONX2DBLCLICK, msg, wParam, lParam);
			return 0;
		}
		case WM_XBUTTONDOWN: {
			if (HIWORD(wParam) == XBUTTON1)
				group->HandleMouseEvent(EVENT_ONX1CLICKDOWN, msg, wParam, lParam);
			else if (HIWORD(wParam) == XBUTTON2)
				group->HandleMouseEvent(EVENT_ONX2CLICKDOWN, msg, wParam, lParam);
			return 0;
		}
		case WM_XBUTTONUP: {
			if (HIWORD(wParam) == XBUTTON1)
				group->HandleMouseEvent(EVENT_ONX1CLICKUP, msg, wParam, lParam);
			else if (HIWORD(wParam) == XBUTTON2)
				group->HandleMouseEvent(EVENT_ONX2CLICKUP, msg, wParam, lParam);
			return 0;
		}
		case WM_RBUTTONDBLCLK: {
			group->HandleMouseEvent(EVENT_ONRIGHTDBLCLICK, msg, wParam, lParam);
			return 0;
		}
		case WM_RBUTTONDOWN: {
			group->HandleMouseEvent(EVENT_ONRIGHTCLICKDOWN, msg, wParam, lParam);
			return 0;
		}
		case WM_RBUTTONUP: {
			group->HandleMouseEvent(EVENT_ONRIGHTCLICKUP, msg, wParam, lParam);
			return 0;
		}
	} // switch(msg)
	return CallWindowProc(group->wpOrigListView_Proc, hListView, msg, wParam, lParam);
}